﻿using System;
using System.IO;
using System.Net;
using System.Data;
using System.Xml;
namespace Joycode.Reader.Framework
{
	/// <summary>
	/// Summary description for RemoteData.
	/// </summary>
	public class RemoteData
	{
		private DataSet set;
		private int index;
		public RemoteData()
		{
			this.set = Manager.GetManager().Data;
		}
		public int Index
		{
			set
			{
				this.index = value;
			}
		}
		public event RemoteUpdateEventHandler RemoteUpdate;
		private void OnRemoteUpdate(UpdateState state)
		{
			if (this.RemoteUpdate != null)
				this.RemoteUpdate(this, new RemoteUpdateEventArgs(state));
		}
		public void Update()
		{
			this.OnRemoteUpdate(new UpdateState("准备同步",false,false));
			DataRow row = set.Tables["BLOGS"].Select(String.Format("id={0}", this.index))[0];
			string rssurl=row["rssurl"].ToString();
			this.LoadDataFromUrl(rssurl, row);
			//DateTime lastModified = 
			row.AcceptChanges();
			LocalData.SaveData(this.set, Manager.GetManager().State.StoreFile);
		}
		private void LoadDataFromUrl(string url,DataRow row)
		{
			DateTime lastModified = DateTime.Parse(row["lastmodified"].ToString());
			Stream stream = null;
			
			WebRequest wRequest = HttpWebRequest.Create(new Uri(url));
			//wRequest.Timeout =1000;// this.Timeout;
			//if (this.Proxy != null) wRequest.Proxy = this.Proxy;
			WebResponse wResponse = null;
			try
			{
				this.OnRemoteUpdate(new UpdateState("建立连接", false, false));
				wResponse = wRequest.GetResponse();
				System.Net.WebHeaderCollection headers = wResponse.Headers;
				DateTime UpdateTime = DateTime.Parse(headers["Last-Modified"]);
				if (UpdateTime > lastModified)
				{
					stream = wResponse.GetResponseStream();
					this.OnRemoteUpdate(new UpdateState("读取内容", false, false));
					StreamReader reader = new StreamReader(stream, System.Text.Encoding.UTF8);
					string content = reader.ReadToEnd();
					this.ParseContent(content);
					reader.Close();
					row["LastModified"] = UpdateTime;
					row.AcceptChanges();
					this.OnRemoteUpdate(new UpdateState("同步完成", true,true));
				}
				else
				{
					this.OnRemoteUpdate(new UpdateState("无更新",true, false));
				}

			}
			catch (WebException)
			{
				this.OnRemoteUpdate(new UpdateState("连接超时", true, false));

			}
			catch 
			{
				this.OnRemoteUpdate(new UpdateState("未知异常", true, false));
			}
			finally
			{
				if (wResponse != null)
					wResponse.Close();
			}

		}
		private void ParseContent(string content)
		{
			this.OnRemoteUpdate(new UpdateState("开始分析", false, false));
			XmlDocument document = new XmlDocument();
			document.LoadXml(content);
			XmlNode root = document.FirstChild;
			if (root.Name.Equals("rss"))
			{
				this.ParseRssContent(document);

			}
		}
		private void ParseRssContent(XmlDocument document)
		{
			this.OnRemoteUpdate(new UpdateState("分析RSS", false, false));
			string name = document.GetElementsByTagName("title")[0].InnerText;
			string url = document.GetElementsByTagName("link")[0].InnerText;
			set.Tables["BLOGS"].Select("id=" + this.index)[0]["title"] = name;
			foreach(DataRow row in set.Tables["ITEMS"].Select("blogid="+this.index))
			{
				row.Delete();
			}
			set.AcceptChanges();
			XmlNodeList lists = document.GetElementsByTagName("item");
			foreach (XmlNode node in lists)
			{
				DataRow row = set.Tables["ITEMS"].NewRow();
				row["BLOGID"] = this.index;
				row["unread"] = true;
				foreach (XmlNode child in node.ChildNodes)
				{
					switch (child.Name)
					{
						case "title":
							row["title"] = child.InnerText;
							break;
						case "link":
							row["link"]=child.InnerText;
							break;
						case "pubDate":
							row["pubdate"] = DateTime.Parse(child.InnerText);
							break;
						case "description":
							row["content"] = child.InnerText;
							break;
						
						case "author":
							row["author"] = child.InnerText;
							break;
						case "dc:creator":
							row["author"] = child.InnerText;
							break;
					}
					

				}
				set.Tables["ITEMS"].Rows.Add(row);

				
			}
			set.AcceptChanges();
		}
	}
}
